@@ -2,25 +2,14 @@ from StringIO import StringIO
 import pickle
 import sys
 import gc
+import copy
 from os import path
 from numpy.testing import *
+from numpy.testing.utils import _assert_valid_refcount
 import numpy as np
 
 rlevel = 1
 
-def assert_valid_refcount(op):
-    a = np.arange(100 * 100)
-    b = np.arange(100*100).reshape(100, 100)
-    c = b
-
-    i = 1
-
-    rc = sys.getrefcount(i)
-    for j in range(15):
-        d = op(b,c)
-
-    assert(sys.getrefcount(i) >= rc)
-
 class TestRegression(TestCase):
     def test_invalid_round(self,level=rlevel):
         """Ticket #3"""
@@ -41,16 +30,6 @@ class TestRegression(TestCase):
         f.close()
         assert_array_equal(a,b)
 
-    def test_masked_array_create(self,level=rlevel):
-        """Ticket #17"""
-        x = np.ma.masked_array([0,1,2,3,0,4,5,6],mask=[0,0,0,1,1,1,0,0])
-        assert_array_equal(np.ma.nonzero(x),[[1,2,6,7]])
-
-    def test_poly1d(self,level=rlevel):
-        """Ticket #28"""
-        assert_equal(np.poly1d([1]) - np.poly1d([1,0]),
-                     np.poly1d([-1,1]))
-
     def test_typeNA(self,level=rlevel):
         """Ticket #31"""
         assert_equal(np.typeNA[np.int64],'Int64')
@@ -106,18 +85,6 @@ class TestRegression(TestCase):
         """Ticket #60"""
         x = np.bool_(1)
 
-    def test_masked_array(self,level=rlevel):
-        """Ticket #61"""
-        x = np.ma.array(1,mask=[1])
-
-    def test_mem_masked_where(self,level=rlevel):
-        """Ticket #62"""
-        from numpy.ma import masked_where, MaskType
-        a = np.zeros((1,1))
-        b = np.zeros(a.shape, MaskType)
-        c = masked_where(b,a)
-        a-c
-
     def test_indexing1(self,level=rlevel):
         """Ticket #64"""
         descr = [('x', [('y', [('z', 'c16', (2,)),]),]),]
@@ -137,11 +104,6 @@ class TestRegression(TestCase):
         x = np.array([1+2j])
         assert_almost_equal(x**(-1), [1/(1+2j)])
 
-    def test_kron_matrix(self,level=rlevel):
-        """Ticket #71"""
-        x = np.matrix('[1 0; 1 0]')
-        assert_equal(type(np.kron(x,x)),type(x))
-
     def test_scalar_compare(self,level=rlevel):
         """Ticket #72"""
         a = np.array(['test', 'auto'])
@@ -159,36 +121,16 @@ class TestRegression(TestCase):
         ua = np.array([[[ucs_value*ulen]*2]*3]*4, dtype='U%s' % ulen)
         ua2 = ua.newbyteorder()
 
-    def test_matrix_std_argmax(self,level=rlevel):
-        """Ticket #83"""
-        x = np.asmatrix(np.random.uniform(0,1,(3,3)))
-        self.assertEqual(x.std().shape, ())
-        self.assertEqual(x.argmax().shape, ())
-
     def test_object_array_fill(self,level=rlevel):
         """Ticket #86"""
         x = np.zeros(1, 'O')
         x.fill([])
 
-    def test_cov_parameters(self,level=rlevel):
-        """Ticket #91"""
-        x = np.random.random((3,3))
-        y = x.copy()
-        np.cov(x,rowvar=1)
-        np.cov(y,rowvar=0)
-        assert_array_equal(x,y)
-
     def test_mem_dtype_align(self,level=rlevel):
         """Ticket #93"""
         self.failUnlessRaises(TypeError,np.dtype,
                               {'names':['a'],'formats':['foo']},align=1)
 
-    def test_mem_digitize(self,level=rlevel):
-        """Ticket #95"""
-        for i in range(100):
-            np.digitize([1,2,3,4],[1,3])
-            np.digitize([0,1,2,3,4],[1,3])
-
     def test_intp(self,level=rlevel):
         """Ticket #99"""
         i_width = np.int_(0).nbytes*2 - 1
@@ -239,15 +181,6 @@ class TestRegression(TestCase):
         for i in xrange(a.ndim):
             aargmax = a.argmax(i)
 
-    def test_matrix_properties(self,level=rlevel):
-        """Ticket #125"""
-        a = np.matrix([1.0],dtype=float)
-        assert(type(a.real) is np.matrix)
-        assert(type(a.imag) is np.matrix)
-        c,d = np.matrix([0.0]).nonzero()
-        assert(type(c) is np.matrix)
-        assert(type(d) is np.matrix)
-
     def test_mem_divmod(self,level=rlevel):
         """Ticket #126"""
         for i in range(10):
@@ -343,10 +276,6 @@ class TestRegression(TestCase):
         def index_tmp(): tmp[np.array(10)]
         self.failUnlessRaises(IndexError, index_tmp)
 
-    def test_unique_zero_sized(self,level=rlevel):
-        """Ticket #205"""
-        assert_array_equal([], np.unique(np.array([])))
-
     def test_chararray_rstrip(self,level=rlevel):
         """Ticket #222"""
         x = np.chararray((1,),5)
@@ -385,14 +314,6 @@ class TestRegression(TestCase):
         import pickle
         pickle.dumps(np.float)
 
-    def test_masked_array_multiply(self,level=rlevel):
-        """Ticket #254"""
-        a = np.ma.zeros((4,1))
-        a[2,0] = np.ma.masked
-        b = np.zeros((4,2))
-        a*b
-        b*a
-
     def test_swap_real(self, level=rlevel):
         """Ticket #265"""
         assert_equal(np.arange(4,dtype='>c8').imag.max(),0.0)
@@ -404,10 +325,6 @@ class TestRegression(TestCase):
         """Ticket #270"""
         a = np.array([1,'A',None])
 
-    def test_masked_array_repeat(self, level=rlevel):
-        """Ticket #271"""
-        np.ma.array([1],mask=False).repeat(10)
-
     def test_multiple_assign(self, level=rlevel):
         """Ticket #273"""
         a = np.zeros((3,1),int)
@@ -416,11 +333,6 @@ class TestRegression(TestCase):
     def test_empty_array_type(self, level=rlevel):
         assert_equal(np.array([]).dtype, np.zeros(0).dtype)
 
-    def test_void_coercion(self, level=rlevel):
-        dt = np.dtype([('a','f4'),('b','i4')])
-        x = np.zeros((1,),dt)
-        assert(np.r_[x,x].dtype == dt)
-
     def test_void_copyswap(self, level=rlevel):
         dt = np.dtype([('one', '<i4'),('two', '<i4')])
         x = np.array((1,2), dtype=dt)
@@ -502,12 +414,6 @@ class TestRegression(TestCase):
         assert_equal(np.array(x,dtype=np.float32,ndmin=2).ndim,2)
         assert_equal(np.array(x,dtype=np.float64,ndmin=2).ndim,2)
 
-    def test_mem_vectorise(self, level=rlevel):
-        """Ticket #325"""
-        vt = np.vectorize(lambda *args: args)
-        vt(np.zeros((1,2,1)), np.zeros((2,1,1)), np.zeros((1,1,2)))
-        vt(np.zeros((1,2,1)), np.zeros((2,1,1)), np.zeros((1,1,2)), np.zeros((2,2)))
-
     def test_mem_axis_minimization(self, level=rlevel):
         """Ticket #327"""
         data = np.arange(5)
@@ -525,11 +431,6 @@ class TestRegression(TestCase):
         """Ticket #335"""
         np.dtype([('col1', '()i4')])
 
-    def test_mgrid_single_element(self, level=rlevel):
-        """Ticket #339"""
-        assert_array_equal(np.mgrid[0:0:1j],[0])
-        assert_array_equal(np.mgrid[0:0],[])
-
     def test_numeric_carray_compare(self, level=rlevel):
         """Ticket #341"""
         assert_equal(np.array([ 'X' ], 'c'),'X')
@@ -586,20 +487,9 @@ class TestRegression(TestCase):
         np.random.shuffle(b)
         assert_equal(np.sort(b, axis=0),a)
 
-    def test_refcount_vectorize(self, level=rlevel):
-        """Ticket #378"""
-        def p(x,y): return 123
-        v = np.vectorize(p)
-        assert_valid_refcount(v)
-
-    def test_poly1d_nan_roots(self, level=rlevel):
-        """Ticket #396"""
-        p = np.poly1d([np.nan,np.nan,1], r=0)
-        self.failUnlessRaises(np.linalg.LinAlgError,getattr,p,"r")
-
     def test_refcount_vdot(self, level=rlevel):
         """Changeset #3443"""
-        assert_valid_refcount(np.vdot)
+        _assert_valid_refcount(np.vdot)
 
     def test_startswith(self, level=rlevel):
         ca = np.char.array(['Hi','There'])
@@ -616,10 +506,6 @@ class TestRegression(TestCase):
         assert_array_equal(np.divide.accumulate(todivide),
             np.array([2., 4., 16.]))
 
-    def test_mem_polymul(self, level=rlevel):
-        """Ticket #448"""
-        np.polymul([],[1.])
-
     def test_convolve_empty(self, level=rlevel):
         """Convolve should raise an error for empty input array."""
         self.failUnlessRaises(ValueError,np.convolve,[],[1])
@@ -636,18 +522,6 @@ class TestRegression(TestCase):
         assert_equal(np.array("a\x00\x0b\x0c\x00").item(),
                      'a\x00\x0b\x0c')
 
-    def test_mem_string_concat(self, level=rlevel):
-        """Ticket #469"""
-        x = np.array([])
-        np.append(x,'asdasd\tasdasd')
-
-    def test_matrix_multiply_by_1d_vector(self, level=rlevel) :
-        """Ticket #473"""
-        def mul() :
-            np.mat(np.eye(2))*np.ones(2)
-
-        self.failUnlessRaises(ValueError,mul)
-
     def test_junk_in_string_fields_of_recarray(self, level=rlevel):
         """Ticket #483"""
         r = np.array([['abc']], dtype=[('var1', '|S20')])
@@ -700,25 +574,6 @@ class TestRegression(TestCase):
         a = np.arange(3, dtype='>f')
         assert a[a.argmax()] == a.max()
 
-    def test_numeric_random(self, level=rlevel):
-        """Ticket #552"""
-        from numpy.oldnumeric.random_array import randint
-        randint(0,50,[2,3])
-
-    def test_poly_div(self, level=rlevel):
-        """Ticket #553"""
-        u = np.poly1d([1,2,3])
-        v = np.poly1d([1,2,3,4,5])
-        q,r = np.polydiv(u,v)
-        assert_equal(q*v + r, u)
-
-    def test_poly_eq(self, level=rlevel):
-        """Ticket #554"""
-        x = np.poly1d([1,2,3])
-        y = np.poly1d([3,4])
-        assert x != y
-        assert x == x
-
     def test_rand_seed(self, level=rlevel):
         """Ticket #555"""
         for l in np.arange(4):
@@ -730,10 +585,6 @@ class TestRegression(TestCase):
         b = np.array(a,dtype=float)
         del a, b
 
-    def test_mem_insert(self, level=rlevel):
-        """Ticket #572"""
-        np.lib.place(1,1,1)
-
     def test_mem_on_invalid_dtype(self):
         "Ticket #583"
         self.failUnlessRaises(ValueError, np.fromiter, [['12',''],['13','']], str)
@@ -805,17 +656,6 @@ class TestRegression(TestCase):
         y = np.fromstring("\x00\x01\x00\x02", dtype="|S2")
         assert_array_equal(np.sort(x, kind="q"), y)
 
-    def test_hist_bins_as_list(self, level=rlevel):
-        """Ticket #632"""
-        import warnings
-        warnings.simplefilter('ignore', Warning)
-        try:
-            hist,edges = np.histogram([1,2,3,4],[1,2],  new=False)
-            assert_array_equal(hist,[1,3])
-            assert_array_equal(edges,[1,2])
-        finally:
-            warnings.resetwarnings()
-
     def test_copy_detection_zero_dim(self, level=rlevel):
         """Ticket #658"""
         np.indices((0,3,4)).T.reshape(-1,3)
@@ -1021,10 +861,6 @@ class TestRegression(TestCase):
         x.fill(1)
         assert_equal(x, np.array([1], dtype=dtype))
 
-    def test_asfarray_none(self, level=rlevel):
-        """Test for changeset r5065"""
-        assert_array_equal(np.array([np.nan]), np.asfarray([None]))
-
     def test_dot_alignment_sse2(self, level=rlevel):
         """Test for ticket #551, changeset r5140"""
         x = np.zeros((30,40))
@@ -1142,23 +978,6 @@ class TestRegression(TestCase):
         assert( a[0].tolist() == b[0])
         assert( a[1].tolist() == b[1])
 
-    def test_large_fancy_indexing(self, level=rlevel):
-        # Large enough to fail on 64-bit.
-        nbits = np.dtype(np.intp).itemsize * 8
-        thesize = int((2**nbits)**(1.0/5.0)+1)
-        def dp():
-            n = 3
-            a = np.ones((n,)*5)
-            i = np.random.randint(0,n,size=thesize)
-            a[np.ix_(i,i,i,i,i)] = 0
-        def dp2():
-            n = 3
-            a = np.ones((n,)*5)
-            i = np.random.randint(0,n,size=thesize)
-            g = a[np.ix_(i,i,i,i,i)]
-        self.failUnlessRaises(ValueError, dp)
-        self.failUnlessRaises(ValueError, dp2)
-
     def test_char_array_creation(self, level=rlevel):
         a = np.array('123', dtype='c')
         b = np.array(['1','2','3'])
@@ -1248,12 +1067,174 @@ class TestRegression(TestCase):
                     assert np.all(z == 0)
                     assert z.shape == (m, n)
 
-    def test_fromiter_comparison(self, level=rlevel):
+    def test_zeros(self):
+        """Regression test for #1061."""
+        # Set a size which cannot fit into a 64 bits signed integer
+        sz = 2 ** 64
+        good = 'Maximum allowed dimension exceeded'
+        try:
+            np.empty(sz)
+        except ValueError, e:
+            if not str(e) == good:
+                self.fail("Got msg '%s', expected '%s'" % (e, good))
+        except Exception, e:
+            self.fail("Got exception of type %s instead of ValueError" % type(e))
+
+    def test_huge_arange(self):
+        """Regression test for #1062."""
+        # Set a size which cannot fit into a 64 bits signed integer
+        sz = 2 ** 64
+        good = 'Maximum allowed size exceeded'
+        try:
+            a = np.arange(sz)
+            self.failUnless(np.size == sz)
+        except ValueError, e:
+            if not str(e) == good:
+                self.fail("Got msg '%s', expected '%s'" % (e, good))
+        except Exception, e:
+            self.fail("Got exception of type %s instead of ValueError" % type(e))
+
+    def test_fromiter_bytes(self):
         """Ticket #1058"""
         a = np.fromiter(range(10), dtype='b')
         b = np.fromiter(range(10), dtype='B')
         assert np.alltrue(a == np.array([0,1,2,3,4,5,6,7,8,9]))
         assert np.alltrue(b == np.array([0,1,2,3,4,5,6,7,8,9]))
 
+    def test_array_from_sequence_scalar_array(self):
+        """Ticket #1078: segfaults when creating an array with a sequence of 0d
+        arrays."""
+        a = np.ones(2)
+        b = np.array(3)
+        assert_raises(ValueError, lambda: np.array((a, b)))
+
+        t = ((1,), np.array(1))
+        assert_raises(ValueError, lambda: np.array(t))
+
+    def test_array_too_big(self):
+        """Ticket #1080."""
+        assert_raises(ValueError, np.zeros, [2**10]*10)
+
+    def test_dtype_keyerrors_(self):
+        """Ticket #1106."""
+        dt = np.dtype([('f1', np.uint)])
+        assert_raises(KeyError, dt.__getitem__, "f2")
+        assert_raises(IndexError, dt.__getitem__, 1)
+        assert_raises(ValueError, dt.__getitem__, 0.0)
+
+    def test_lexsort_buffer_length(self):
+        """Ticket #1217, don't segfault."""
+        a = np.ones(100, dtype=np.int8)
+        b = np.ones(100, dtype=np.int32)
+        i = np.lexsort((a[::-1], b))
+        assert_equal(i, np.arange(100, dtype=np.int))
+
+    def test_object_array_to_fixed_string(self):
+        """Ticket #1235."""
+        a = np.array(['abcdefgh', 'ijklmnop'], dtype=np.object_)
+        b = np.array(a, dtype=(np.string_, 8))
+        assert_equal(a, b)
+        c = np.array(a, dtype=(np.string_, 5))
+        assert_equal(c, np.array(['abcde', 'ijklm']))
+        d = np.array(a, dtype=(np.string_, 12))
+        assert_equal(a, d)
+        e = np.empty((2, ), dtype=(np.string_, 8))
+        e[:] = a[:]
+        assert_equal(a, e)
+
+    def test_unicode_to_string_cast(self):
+        """Ticket #1240."""
+        a = np.array([[u'abc', u'\u03a3'], [u'asdf', u'erw']], dtype='U')
+        def fail():
+            b = np.array(a, 'S4')
+        self.failUnlessRaises(UnicodeEncodeError, fail)
+
+    def test_mixed_string_unicode_array_creation(self):
+        a = np.array(['1234', u'123'])
+        assert a.itemsize == 16
+        a = np.array([u'123', '1234'])
+        assert a.itemsize == 16
+        a = np.array(['1234', u'123', '12345'])
+        assert a.itemsize == 20
+        a = np.array([u'123', '1234', u'12345'])
+        assert a.itemsize == 20
+        a = np.array([u'123', '1234', u'1234'])
+        assert a.itemsize == 16
+
+    def test_misaligned_objects_segfault(self):
+        """Ticket #1198 and #1267"""
+        a1 = np.zeros((10,), dtype='O,c')
+        a2 = np.array(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j'], 'S10')
+        a1['f0'] = a2
+        r = repr(a1)
+        np.argmax(a1['f0'])
+        a1['f0'][1] = "FOO"
+        a1['f0'] = "FOO"
+        a3 = np.array(a1['f0'], dtype='S')
+        np.nonzero(a1['f0'])
+        a1.sort()
+        a4 = copy.deepcopy(a1)
+
+    def test_misaligned_scalars_segfault(self):
+        """Ticket #1267"""
+        s1 = np.array(('a', 'Foo'), dtype='c,O')
+        s2 = np.array(('b', 'Bar'), dtype='c,O')
+        s1['f1'] = s2['f1']
+        s1['f1'] = 'Baz'
+
+    def test_misaligned_dot_product_objects(self):
+        """Ticket #1267"""
+        # This didn't require a fix, but it's worth testing anyway, because
+        # it may fail if .dot stops enforcing the arrays to be BEHAVED
+        a = np.array([[(1, 'a'), (0, 'a')], [(0, 'a'), (1, 'a')]], dtype='O,c')
+        b = np.array([[(4, 'a'), (1, 'a')], [(2, 'a'), (2, 'a')]], dtype='O,c')
+        np.dot(a['f0'], b['f0'])
+
+    def test_byteswap_complex_scalar(self):
+        """Ticket #1259"""
+        z = np.array([-1j], '<c8')
+        x = z[0] # always native-endian
+        y = x.byteswap()
+        if x.dtype.byteorder == z.dtype.byteorder:
+            # little-endian machine
+            assert_equal(x, np.fromstring(y.tostring(), dtype='>c8'))
+        else:
+            # big-endian machine
+            assert_equal(x, np.fromstring(y.tostring(), dtype='<c8'))
+
+    def test_structured_arrays_with_objects(self):
+        """Ticket #1299"""
+        stra = 'aaaa'
+        strb = 'bbbb'
+        x = np.array([[(0,stra),(1,strb)]], 'i8,O')
+        x[x.nonzero()] = x.ravel()[:1]
+        assert x[0,1] == x[0,0]
+
+    def test_structured_arrays_with_objects2(self):
+        """Ticket #1299 second test"""
+        stra = 'aaaa'
+        strb = 'bbbb'
+        numb = sys.getrefcount(strb)
+        numa = sys.getrefcount(stra)
+        x = np.array([[(0,stra),(1,strb)]], 'i8,O')
+        x[x.nonzero()] = x.ravel()[:1]
+        assert sys.getrefcount(strb) == numb
+        assert sys.getrefcount(stra) == numa + 2
+        
+    def test_signed_integer_division_overflow(self):
+        """Ticket #1317."""
+        def test_type(t):
+            min = np.array([np.iinfo(t).min])
+            min /= -1
+
+        for t in (np.int8, np.int16, np.int32, np.int64, np.int, np.long):
+            test_type(t)
+
+
 if __name__ == "__main__":
     run_module_suite()
+    def test_duplicate_title_and_name(self):
+        """Ticket #1254"""
+        def func():
+            x = np.dtype([(('a', 'a'), 'i'), ('b', 'i')])
+        self.failUnlessRaises(ValueError, func)
